/*
 * Operations and their invocation.
 *
 * Copyright (C) 2022 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include <ext2fs/ext2fs.h>

/* Invocation support. */

enum op_results
{
    OP_SUCCESS = 0,
    OP_FAILED = 1,
    OP_UNKNOWN = 2,
};

int handle_op_result(const char *operation, enum op_results op_result);
enum op_results run_operation(ext2_filsys fs, const char *operation, int argc, char *argv[]);

/* Operations exposed by the program. */

typedef int (*op_sig)(ext2_filsys, int, char *[]);

struct operation
{
    const char *name;
    op_sig fn;
};

int copy_in(ext2_filsys fs, int argc, char *argv[]);
int copy_out(ext2_filsys fs, int argc, char *argv[]);
int list(ext2_filsys fs, int argc, char *argv[]);
int make_dirs(ext2_filsys fs, int argc, char *argv[]);
int remove_dirs(ext2_filsys fs, int argc, char *argv[]);
int remove_non_dirs(ext2_filsys fs, int argc, char *argv[]);
int run_script(ext2_filsys fs, int argc, char *argv[]);
int stat_objects(ext2_filsys fs, int argc, char *argv[]);

/* vim: tabstop=4 expandtab shiftwidth=4
*/
