/*
 * Stack support for new tasks and threads.
 *
 * Copyright (C) 2022 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include <l4/re/env.h>
#include <l4/re/l4aux.h>

#include <exec/segment.h>



/* A stack abstraction. */

class Stack
{
protected:

  /* Auxiliary vector key-value pair. */

  struct auxv_entry
  {
    l4_umword_t key, value;
  };

  /* Program segment holding the stack contents. */

  Segment &_segment;

  /* Next element pointer. */

  l4_umword_t *_element;

  /* Stack section properties. */

  l4_addr_t _caps;
  char *_arg_top, *_env_top;
  char **_argv;
  char *_auxv_end;
  int _env_entries;

  /* L4Re auxiliary and environment regions. */

  l4re_aux_t *_aux;
  l4re_env_t *_env;

  /* Stack pointer start address. */

  l4_addr_t _start;

  /* Internal stack population methods. */

  void push_cap_entries(l4re_env_cap_entry_t *entries);

  void push_string(char *s);

  void push_env(char *envp[]);

  void push_args(int argc, char *argv[]);

  void push_l4re_aux();

  void push_l4re_env();

  void push_auxv();

  offset_t write_address(char *arg, char **addr, char *s);

  void push_envp(char *envp[]);

  void push_argv(int argc, char *argv[]);

  l4_addr_t align_stack();

public:
  explicit Stack(Segment &segment);

  void set_l4re_aux(l4re_aux_t *aux);

  void set_l4re_env(l4re_env_t *env);

  void populate(int argc, char *argv[], char *envp[]);

  l4_addr_t start_address();
};

/* vim: tabstop=2 expandtab shiftwidth=2
*/
