/*
 * A page queue whose users take turns to access pages.
 *
 * Copyright (C) 2021, 2022 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include <condition_variable>
#include <mutex>

#include <fsserver/page_queue_common.h>



/* A queue of issued pages. */

class PageQueueShared : public PageQueueCommon
{
protected:
    Queue _queue;
    Positions _positions;

    std::mutex _lock;
    std::condition_variable _counter;

    virtual bool _pop(QueueEntry *entry);

public:
    virtual void close(Memory *memory);

    virtual void pop(PageOwner **owner, Flexpage **flexpage);

    virtual void push(PageOwner *owner, Flexpage *flexpage);

    virtual void push_front(PageOwner *owner, Flexpage *flexpage);

    virtual bool remove(PageOwner *owner, Flexpage *flexpage);
};

// vim: tabstop=4 expandtab shiftwidth=4
